/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:
 known Problems: none
 Version:        12.06.2008
 Description:    Timer Routine fr LED Matrix Bilderzeugung

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/
#include "display.h"

volatile unsigned char row_counter = 0;
volatile unsigned char col_counter = 2;
volatile unsigned char pixel_buffer[5] = {0xff,0xff,0xff,0xff,0xff};

//----------------------------------------------------------------------------
//Diese Routine startet und inizialisiert den Timer und Display Ports
void display_init (void)
{
    TCCR1B |= (1<<CS10);
    TCNT1 = 0;
    OCR1A = 50;
    
    TIMSK |= (1 << OCIE1A);
    
    DDRD |= (1<<PD6)|(1<<PD5)|(1<<PD4)|(1<<PD3)|(1<<PD2);  
    DDRB |= (1<<PB6)|(1<<PB5)|(1<<PB4)|(1<<PB3)|(1<<PB2)|(1<<PB1)|(1<<PB0);
    
    
    #if MODE_TA_TC
    PORTB|= (1<<PB6)|(1<<PB5)|(1<<PB4)|(1<<PB3)|(1<<PB2)|(1<<PB1)|(1<<PB0);
    PORTD|= (1<<PD6)|(1<<PD5)|(1<<PD4)|(1<<PD3)|(1<<PD2);
    #else
    PORTB&=~((1<<PB6)|(1<<PB5)|(1<<PB4)|(1<<PB3)|(1<<PB2)|(1<<PB1)|(1<<PB0));
    PORTD&=~((1<<PD6)|(1<<PD5)|(1<<PD4)|(1<<PD3)|(1<<PD2));
    #endif
};

//----------------------------------------------------------------------------
//Timer Interrupt
ISR (TIMER1_COMPA_vect)
{ 
    #if MODE_TA_TC
        PORTB = 0x00;
    #else
        PORTB = 0xff;
    #endif 
    
    #if MULTI_SINGLE_LED
        OCR1A = 500;
    #else
        if (OCR1A != 5000)
        {
            OCR1A = 5000;
            return;
        }
    #endif
            
    TCNT1 = 0;
    if((++row_counter)>6)
    {
        row_counter = 0;           
        if((++col_counter)>4) col_counter = 0;
        
        #if MODE_TA_TC
        PORTD = 0xFF & (~(1<<(2+col_counter)));
        #else
        PORTD = (1<<(2+col_counter));
        #endif
    }
    
    #if MULTI_SINGLE_LED
        if (pixel_buffer[col_counter] & (1<<row_counter))
        {
            #if MODE_TA_TC
                PORTB = (1<<row_counter);
            #else
                PORTB = 0xff & (~(1<<row_counter));
            #endif
        }
    #else  
        unsigned char led_on_counter = 0;
 
        #if MODE_TA_TC
            PORTB = 0x00;
        #else
            PORTB = 0xff;
        #endif
        for (row_counter = 0;row_counter<7;row_counter++)
        {
            //LED_ON
            if (pixel_buffer[col_counter] & (1<<row_counter))
            {
                #if MODE_TA_TC
                    PORTB |= (1<<row_counter);
                #else
                    PORTB &=~(1<<row_counter);
                #endif
                led_on_counter++;
            }
        }
        if (led_on_counter > 0)
        {
            OCR1A = 1500 + (led_on_counter * 500);
        }

    #endif
}
