/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    RS232 Routinen

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
------------------------------------------------------------------------------*/

#include "usart.h"

volatile unsigned char rx_buffer_counter = 0;
volatile unsigned char display_mode = 0;
volatile unsigned char display_swap = 0;

//----------------------------------------------------------------------------
//Init serielle Schnittstelle
void usart_init(unsigned long baudrate) 
{ 
  	//Enable TXEN im Register UCR TX-Data Enable
	UCR  = (1 << TXEN | 1 << RXEN | 1<< RXCIE);
    //USR |= (1 << U2X);
	//Teiler wird gesetzt 
	UBRR=(SYSCLK / (baudrate * 16L) - 1);
}

//----------------------------------------------------------------------------
//Routine fr die Serielle Ausgabe eines Zeichens
void usart_write_char(char c)
{
	//Warten solange bis Zeichen gesendet wurde
	while(!(USR & (1<<UDRE)));
	//Ausgabe des Zeichens
	UDR = c;
	return;
}

//----------------------------------------------------------------------------
//Empfang eines Zeichens
ISR (USART_RX)
{
    unsigned char rx_char = UDR;
    
    if (rx_char > 128)
    {
        if(rx_char == 255)
        {
            rx_buffer_counter = 0;
            usart_write_char(rx_char);
            return;
        }
        if(rx_char == 254)
        {
            display_mode = 0x01;
            usart_write_char(rx_char);
            return;
        }
        if(rx_char == 253)
        {
            display_mode = 0x00;
            usart_write_char(rx_char);
            return;
        }
        if(rx_char == 252)
        {
            display_swap = 0x01;
            return;
        }
        if(rx_char == 251)
        {
            display_swap = 0x00;
            return;
        }
    }
    
    if(display_mode)
    {
        unsigned char tmp = 0;
        if (display_swap)
        {
            tmp |= (rx_char&0x01)<<6;
            tmp |= (rx_char&0x02)<<4;
            tmp |= (rx_char&0x04)<<2;
            tmp |= (rx_char&0x08);
            tmp |= (rx_char&0x10)>>2;
            tmp |= (rx_char&0x20)>>4;
            tmp |= (rx_char&0x40)>>6;
        }
        else
        {
            tmp = rx_char;
        }
    
        usart_write_char(pixel_buffer[4]);
        pixel_buffer[4] = pixel_buffer[3];
        pixel_buffer[3] = pixel_buffer[2];
        pixel_buffer[2] = pixel_buffer[1];
        pixel_buffer[1] = pixel_buffer[0];
        pixel_buffer[0] = tmp;
    }
    else
    { 
        if (rx_buffer_counter < 5)
        {
            pixel_buffer[rx_buffer_counter++] = rx_char;
        }
        else
        {
            usart_write_char(rx_char);
        }
    }
}




